# -*- coding: utf-8 -*-

#  /_/ _  _ _  /_      _
# / / /_'/ /_//\ /_/. /_//_/
#                    /   _/


"""
heroku3.py
~~~~~~~~~

:copyright: (c) 2013 by Heroku.
:license: MIT, see LICENSE for more details.
"""

# Meta.

__title__ = "heroku3"
__author__ = "Heroku"
__copyright__ = "Copyright 2013 Heroku"
__license__ = "MIT"

__version__ = "5.2.0"

# Module namespace.

# Project libraries
from .core import from_key  # noqa



# -*- coding: utf-8 -*-

"""
heroku3.models
~~~~~~~~~~~~~

This module contains the models that comprise the Heroku API.
"""

import sys
from pprint import pprint  # noqa

# Project libraries
from ..helpers import to_python

if sys.version_info > (3, 0):
    from urllib.parse import quote
else:
    from urllib import quote  # noqa


class BaseResource(object):

    _arrays = {}
    _bools = []
    _dicts = []
    _dates = []
    _ints = []
    _map = {}
    _pks = []
    _strs = []
    order_by = "id"

    def __init__(self):
        self._bootstrap()
        self._h = None
        super(BaseResource, self).__init__()

    def __repr__(self):
        return "<resource '{0}'>".format(self._id)

    def _bootstrap(self):
        """Bootstraps the model object based on configured values."""
        for attr in self._keys():
            setattr(self, attr, None)

    @classmethod
    def _keys(cls):
        """The list of user declared attributes for this model.

        Returns:
            List[str]: the names of the user declared attributes.
        """
        # self._pks is (should be, don't see where this is enforeced) a subset
        # of the set below, hence there is no need to include it.
        return (
            list(cls._arrays.keys())
            + cls._bools
            + cls._dates
            + cls._dicts
            + cls._ints
            + list(cls._map.keys())
            + cls._strs
        )

    @property
    def _id(self):
        try:
            return getattr(self, self._pks[0])
        except IndexError:
            return None

    @property
    def _ids(self):
        """The list of primary keys to validate against."""
        for pk in self._pks:
            yield getattr(self, pk)

        for pk in self._pks:

            try:
                yield str(getattr(self, pk))
            except ValueError:
                pass

    def dict(self):
        d = dict()
        for k in self._keys():
            d[k] = self.__dict__.get(k)

        return d

    def change_connection(self, h):
        self._h = h
        return self

    @classmethod
    def new_from_dict(cls, d, h=None, **kwargs):

        d = to_python(
            obj=cls(),
            in_dict=d,
            strs=cls._strs,
            ints=cls._ints,
            dates=cls._dates,
            bools=cls._bools,
            dicts=cls._dicts,
            objects=cls._map,
            arrays=cls._arrays,
            _h=h,
        )

        d.__dict__.update(kwargs)

        return d


class Price(BaseResource):
    """Heroku Price."""

    _strs = ["cents", "unit"]
    _pks = ["cents"]

    def __init__(self):
        self.app = None
        super(Price, self).__init__()

    def __repr__(self):
        return "<price '{0} per {1}'>".format(self.cents, self.unit)


class Plan(BaseResource):
    """Heroku Addon."""

    _strs = ["id", "name", "description", "state"]
    _pks = ["name", "id"]
    _map = {"price": Price}
    _dates = ["created_at", "updated_at"]

    def __repr__(self):
        return "<Plan '{0}'>".format(self.name)


class Stack(BaseResource):
    """Heroku Stack."""

    _strs = ["id", "name"]
    _pks = ["id", "name"]

    def __init__(self):
        self.app = None
        super(Stack, self).__init__()

    def __repr__(self):
        return "<stack '{0}'>".format(self.name)


class User(BaseResource):
    """Heroku User."""

    _strs = ["id", "email"]
    _pks = ["id", "email"]

    def __init__(self):
        self.app = None
        super(User, self).__init__()

    def __repr__(self):
        return "<user '{0}'>".format(self.email)


class Organization(BaseResource):
    """Heroku Organization."""

    _strs = ["id", "name"]
    _pks = ["id", "name"]

    def __init__(self):
        self.app = None
        super(Organization, self).__init__()

    def __repr__(self):
        return "<organization '{0}'>".format(self.name)


class Team (BaseResource):
    """Heroku Team."""

    _strs = ["id", "name"]
    _pks = ["id", "name"]

    def __init__(self):
        self.app = None
        super(Team, self).__init__()

    def __repr__(self):
        return "<Team '{0}'>".format(self.name)


class Space(BaseResource):
    """Heroku Space."""

    _strs = ["id", "name"]
    _pks = ["id", "name"]
    _bools = ["shield"]

    def __init__(self):
        self.app = None
        super(Space, self).__init__()

    def __repr__(self):
        return "<Space '{0}'>".format(self.name)


# class Plan(BaseResource):
# """Heroku Addon."""
#
# _strs = ['id', 'name']
# _pks = ['id', 'name']
#
# def __init__(self):
# self.app = None
# super(Plan, self).__init__()

# def __repr__(self):
# return "<plan '{0} {1}'>".format(self.id, self.name)
#


class RateLimit(BaseResource):
    _strs = ["remaining"]
    _bools = []
    _pks = ["remaining"]

    def __init__(self):
        self.app = None
        super(RateLimit, self).__init__()

    def __repr__(self):
        return "<RateLimit '{0}'>".format(self.remaining)



# Project libraries
from . import BaseResource


class Invoice(BaseResource):
    _dates = ["created_at", "updated_at"]
    _ints = ["number", "credits_total", "total", "charges_total"]
    _strs = ["id"]
    _pks = ["id", "number"]

    def __init__(self):
        super(Invoice, self).__init__()

    def __repr__(self):
        return "<invoice '{}-{}'>".format(self.id, self.number)



# Project libraries
from . import BaseResource


class Domain(BaseResource):
    """Heroku Domain."""

    _strs = ["id", "hostname", "cname", "kind", "status", "acm_status"]
    _dates = ["created_at", "updated_at"]
    _pks = ["hostname", "id"]
    order_by = "domain"

    def __init__(self):
        self.app = None
        super(Domain, self).__init__()

    def __repr__(self):
        return "<domain '{0}'>".format(self.hostname)

    def remove(self):
        r = self._h._http_resource(method="DELETE", resource=("apps", self.app.name, "domains", self.hostname))

        r.raise_for_status()

        return r.ok



# Project libraries
from .. import BaseResource


class AccountFeature(BaseResource):
    _strs = ["name", "description", "doc_url", "id"]
    _bools = ["enabled"]
    _dates = ["created_at", "updated_at"]
    _pks = ["id", "name"]

    def __init__(self):
        self.app = None
        super(AccountFeature, self).__init__()

    def __repr__(self):
        return "<account_feature '{0}'>".format(self.name)

    def update(self, enabled):
        r = self._h._http_resource(
            method="POST",
            resource=("account", "features", self.id),
            data=self._h._resource_serialize({"enabled": enabled}),
        )
        r.raise_for_status()
        return r.ok

    def enable(self):
        return self.update(True)

    def disable(self):
        return self.update(0)



import sys

# Project libraries
from ..key import Key
from .feature import AccountFeature
from ...models import BaseResource

if sys.version_info > (3, 0):
    from urllib.parse import quote
else:
    from urllib import quote  # noqa


class Account(BaseResource):

    _strs = ["email", "id", "name"]
    _bools = ["allow_tracking", "beta", "verified"]
    _pks = ["id"]
    _dates = ["created_at", "last_login", "updated_at"]

    def __repr__(self):
        return "<account '{0}'>".format(self.email)

    def keys(self, **kwargs):
        """The collaborators for this app."""
        return self._h._get_resources(resource=("account", "keys"), obj=Key, app=self, **kwargs)

    def add_key(self, key):
        r = self._h._http_resource(
            method="POST", resource=("account", "keys"), data=self._h._resource_serialize({"public_key": key})
        )
        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return Key.new_from_dict(item, h=self._h)

    def remove_key(self, key_or_fingerprint):
        """Deletes the key."""
        r = self._h._http_resource(method="DELETE", resource=("account", "keys", quote(key_or_fingerprint)))

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return Key.new_from_dict(item, h=self._h)

    def disable_feature(self, id_or_name):
        return self.update_feature(id_or_name, 0)

    def enable_feature(self, id_or_name):
        return self.update_feature(id_or_name, True)

    def update_feature(self, id_or_name, enabled):

        payload = {"enabled": enabled}
        r = self._h._http_resource(
            method="PATCH", resource=("account", "features", id_or_name), data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return AccountFeature.new_from_dict(item, h=self._h, app=self)

    def change_password(self, current_password, new_password):
        r = self._h._http_resource(
            method="PUT",
            resource=("account", "password"),
            data=self._h._resource_serialize({"current_password": current_password, "new_password": new_password}),
        )
        r.raise_for_status()
        return r.ok
